% Code for Bonhomme and Weidner (2021), "Minimizing Sensitivity to Model Misspecification"
% This code reproduces Figures S1 in the appendix
% (up to some randomness due to different seeds)

clear
clc

close all

N=100000;

% Covariates X
NX=20;
pi_X=(1/NX:1/NX:(NX-1)/NX);

X=zeros(N,1);

UX=rand(N,1);

for kX=1:NX
    if kX==1
        indX=(UX<pi_X(1));
    elseif kX>1 && kX<NX
        indX=(UX>=pi_X(kX-1)).*(UX<pi_X(kX));
    elseif kX==NX
        indX=(UX>=pi_X(NX-1));
    end
    
    X(indX==1)=(kX-1/2)/NX;
end

Xtot=[X ones(N,1)];

[~,dimbeta]=size(Xtot);

beta=[2;-1];

par=beta;

% discretization
KK=5000;

a_kk=sort(randn(KK,1),1);
Mat=diag(normcdf(a_kk(2:end))-normcdf(a_kk(1:end-1)));
D2=Mat;


invD2=inv(D2);

lambdagrid=(1:1:3)';
[nlambdagrid,~]=size(lambdagrid);

% dgp
UU=rand(N,1);
A=(UU>2/3).*(2+.5*randn(N,1))+(UU<2/3).*(-1+.3*randn(N,1));
std1=std(A);
A=A/std(A);

Y=(Xtot*beta+A>=0);

S1=Xtot.*normpdf(Xtot*par).*(1-normcdf(Xtot*par))./(normcdf(Xtot*par).*(1-normcdf(Xtot*par)));
S0=Xtot.*normpdf(Xtot*par).*(0-normcdf(Xtot*par))./(normcdf(Xtot*par).*(1-normcdf(Xtot*par)));
S=[S1;S0];

H=(Xtot.*(normpdf(Xtot*par)).^2./(normcdf(Xtot*par).*(1-normcdf(Xtot*par))))'*Xtot/N;

FF1=(-Xtot*par<a_kk(1:end-1)').*(normcdf(a_kk(2:end))-normcdf(a_kk(1:end-1)))'./(normcdf(Xtot*par))+...
    ((-Xtot*par>a_kk(1:end-1)').*(-Xtot*par<=a_kk(2:end)')).*(normcdf(a_kk(2:end))'-normcdf(-Xtot*par))./(normcdf(Xtot*par));

FF0=(-Xtot*par>a_kk(2:end)').*(normcdf(a_kk(2:end))'-normcdf(a_kk(1:end-1))')./(1-normcdf(Xtot*par))+...
    (-Xtot*par>a_kk(1:end-1)').*(-Xtot*par<=a_kk(2:end)').*(normcdf(-Xtot*par)-normcdf(a_kk(1:end-1))')./(1-normcdf(Xtot*par));

Vect=((normcdf(Xtot*par).*FF1)'*S1/N+((1-normcdf(Xtot*par)).*FF0)'*S0/N);

HHtilde2=(normcdf(Xtot*par).*FF1)'*FF1/N+((1-normcdf(Xtot*par)).*FF0)'*FF0/N-Vect*inv(H)*Vect';

[V,D]=eigs(invD2*HHtilde2,NX-1);

aaa=(a_kk(1:end-1)+a_kk(2:end))/2;

pi_0=1/3*(1/(.5/std1))*normpdf((aaa-2/std1)/(.5/std1))+2/3*(1/(.3/std1))*normpdf((aaa+1/std1)/(.3/std1));

pi_star=normpdf(aaa);
proj=V\((pi_0-pi_star)./pi_star);

% Figure S1
if NX==20
    figure
    plot(aaa,V(:,2),'--','color','k','Linewidth',3)
    hold on
    plot(aaa,V(:,3),'-.','color','r','Linewidth',3)
    hold on
    plot(aaa,V(:,4),':','color','y','Linewidth',3)
    axis([-2 2.5 -.03 .03])
    xlabel('latent variable A','FontSize',15)
    ylabel('eigenfunction','FontSize',15)
    hold off
    
    figure
    plot(aaa,pi_0,'-','color','b','Linewidth',3)
    hold on
    plot(aaa,pi_star,'-','color','g','Linewidth',3)
    hold on
    plot(aaa,pi_star+V(:,1:2)*proj(1:2).*pi_star,'--','color','k','Linewidth',3)
    hold on
    plot(aaa,pi_star+V(:,1:3)*proj(1:3).*pi_star,'-.','color','r','Linewidth',3)
    hold on
    plot(aaa,pi_star+V(:,1:4)*proj(1:4).*pi_star,':','color','y','Linewidth',3)
    axis([-2 2.5 -.5 1.5])
    xlabel('latent variable A','FontSize',15)
    ylabel('density','FontSize',15)
    hold off
elseif NX==4
    
    figure
    plot(aaa,V(:,2),'--','color','k','Linewidth',3)
    hold on
    plot(aaa,V(:,3),'-.','color','r','Linewidth',3)
    axis([-2 2.5 -.03 .03])
     xlabel('latent variable A','FontSize',15)
    ylabel('eigenfunction','FontSize',15)
       hold off
    
    figure
    plot(aaa,pi_0,'-','color','b','Linewidth',3)
    hold on
    plot(aaa,pi_star,'-','color','g','Linewidth',3)
    hold on
    plot(aaa,pi_star+V(:,1:2)*proj(1:2).*pi_star,'--','color','k','Linewidth',3)
    hold on
    plot(aaa,pi_star+V(:,1:3)*proj(1:3).*pi_star,'-.','color','r','Linewidth',3)
    axis([-2 2.5 -.5 1.5])
      xlabel('latent variable A','FontSize',15)
    ylabel('density','FontSize',15)
    hold off
end

