% Code for Bonhomme and Weidner (2021), "Minimizing Sensitivity to Model Misspecification"
% This code reproduces Figures 3

clear
clc

close all

S=100000;

x_grid=(-4:.01:4)';

% loop on epsilon
for epsilon=[.1 1]

% cdf

CDF0=normcdf(x_grid);

% bounds

Avar=randn(S,1);

Deltavar=(Avar+x_grid'>=0);

par=[ones(S,1) Avar Avar.^2]\Deltavar;

Resvar=Deltavar;

CDF1=min(normcdf(x_grid)+sqrt(epsilon)*sqrt(var(Resvar)'),1);

CDF2=max(normcdf(x_grid)-sqrt(epsilon)*sqrt(var(Resvar)'),0);

% logit

CDFlogit=exp(x_grid*(pi/sqrt(3)))./(1+exp(x_grid*(pi/sqrt(3))));

% t with 2.1 degrees of F

tdeg=2.1;

CDFt=tcdf(x_grid*sqrt(tdeg/(tdeg-2)),tdeg);

% density of A in the simulation

CDFs=1/3*normcdf((1.4644*x_grid-2)/.5)+2/3*normcdf((1.4644*x_grid+1)/.3);

% Figure 3
figure
plot(CDF0,CDF0,'-','Color','g','Linewidth',3)
hold on
plot(CDF0,[CDF1 CDF2],'--','Color','b','Linewidth',3)
xlabel('value \Phi(a) according to reference model','FontSize',15)
ylabel('bound in the neighborhood','FontSize',15)
hold off

figure
plot(x_grid,CDF0,'-','Color','g','Linewidth',3)
hold on
plot(x_grid,[CDF1 CDF2],'--','Color','b','Linewidth',3)
xlabel('a value','FontSize',15)
ylabel('CDF','FontSize',15)
hold off

end

A=sort(randn(S,1),1);

pi0=1/3*normpdf((1.4644*A-2)/.5)*1.4644/.5+2/3*normpdf((1.4644*A+1)/.3)*1.4644/.3;
pistar=normpdf(A);
mean((pi0./pistar-1).^2)

pi0=tpdf(A*sqrt(tdeg/(tdeg-2)),tdeg)*sqrt(tdeg/(tdeg-2));
mean((pi0./pistar-1).^2)

CDFlogit=exp(A*(pi/sqrt(3)))./(1+exp(A*(pi/sqrt(3))));
pi0=CDFlogit.*(1-CDFlogit)*(pi/sqrt(3));
mean((pi0./pistar-1).^2)


    