% Code for Bonhomme and Weidner (2021), "Minimizing Sensitivity to Model Misspecification"
% This matlab code replicates the ex post parts of Figure 1 and Table 1
% It uses the data sets produced by "Code_prepare_data"

clear
clc
close all

nbvar=32;
% Load data
% write your path here
data = csv2mat_numeric('data_female.out');
Vec_f=struct2cell(data);
Vec_f=cell2mat(Vec_f);
[Nvar a]=size(Vec_f);
N=Nvar/nbvar;
Mat_f=Vec_f(1:N);
for j=2:nbvar
    Mat_f=[Mat_f Vec_f((j-1)*N+1:j*N)];
end

% write your path here
data = csv2mat_numeric('data_male.out');
Vec_m=struct2cell(data);
Vec_m=cell2mat(Vec_m);
[Nvar a]=size(Vec_m);
N=Nvar/nbvar;
Mat_m=Vec_m(1:N);
for j=2:nbvar
    Mat_m=[Mat_m Vec_m((j-1)*N+1:j*N)];
end

% loop on gender

for female=0:1
    
    if female==1
        Mat=Mat_f;
    else
        Mat=Mat_m;
    end
    
    % introduce the matrix of regressors
    X=[Mat(:,31) Mat(:,32) (Mat(:,10)==13) (Mat(:,10)==14) (Mat(:,10)==15)...
        Mat(:,6) Mat(:,7) Mat(:,16) (Mat(:,18)==98) Mat(:,20) Mat(:,14)];
    
    [N col]=size(X);
    
    X1=[X ones(N,1)];
    
    % load the parameter vectors produced by "Code_prepare_data"
    if female==1
        
        par=[ .00718376    -.0070885   -.42464005   -.65101909   -1.4146266    .01823748 ...
            -.00307543   -.00012721   -.11597747   -.29323347  .19423772    .72366156]';
        
    else
        par=[ .00872072   -.00853673   -.26647581   -.76121597   -1.0955773   -.04132661 ...
            .01548341   -.00007747    .00129391   -.27908219 .24247246    2.2457318]';
        
        
    end
    
    % control indicator
    cont=Mat(:,27);
    
    % poor indicator
    poor=Mat(:,20);
    
    % age
    age=Mat(:,10);
    
    % education indicator
    educ=Mat(:,13);
    
    % highest grade obtained
    grade=Mat(:,14);
    
    % following grade, which affects the subsidy amount
    grade=grade+1;
    
    % PROGRESA IMPACTS
    
    % estimates
    
    if female==1
        disp('females')
    else
        disp('males')
    end
    
    disp('point estimates')
    
    % model-based
    
    disp('experimental estimate')
    
    res_exp=sum(educ.*(1-cont).*poor)/sum((1-cont).*poor)-sum(educ.*(cont).*poor)/sum((cont).*poor);
    disp(res_exp)
    
    % subsidy amounts, from Todd and Wolpin (2006)
    if female==1
        tauvar=(12/52)*(70*(grade==3)+80*(grade==4)+105*(grade==5)+135*(grade==6)+210*(grade==7)+235*(grade==8)+255*(grade==9));
    else
        tauvar=(12/52)*(70*(grade==3)+80*(grade==4)+105*(grade==5)+135*(grade==6)+200*(grade==7)+210*(grade==8)+225*(grade==9));
    end
    
    res=sum((normcdf(X1*par+par(1)*tauvar)-normcdf(X1*par)).*poor)/sum(poor);
    
    disp('model-based')
    disp(res)
    
    % Minimum-MSE estimator
    
    tauvar_tot=tauvar.*(1-cont).*poor;
    
    
    
    X1tau=X1;
    X1tau(:,1)=X1(:,1)+tauvar_tot;
    
    Vect=normpdf(X1tau*par).^2./(normcdf(X1tau*par).*(1-normcdf(X1tau*par)));
    
    
    Heta=((Vect*ones(1,col+1)).*X1tau)'*X1tau/N;
    
    
    V=tauvar_tot.*normpdf(X1tau*par).^2./(normcdf(X1tau*par).*(1-normcdf(X1tau*par)));
    
    delta_theta=sum((normcdf(X1*par+par(1)*tauvar)-normcdf(X1*par)).*poor)/sum(poor);
    
    X1tau1=X1;
    X1tau1(:,1)=X1(:,1)+tauvar;
    
    nabla_eta_delta=(X1tau1'*(poor.*(normpdf(X1*par+par(1)*tauvar)))-X1'*(poor.*(normpdf(X1*par))))/sum(poor);
    
    
    nabla_tilde_theta_delta=(tauvar'*(poor.*(normpdf(X1*par+par(1)*tauvar))))/sum(poor)...
        -mean((V*ones(1,col+1)).*X1tau)*inv(Heta)*nabla_eta_delta;
    
    nabla_eta_logf=((normpdf(X1tau*par).*(educ-normcdf(X1tau*par))./(normcdf(X1tau*par).*...
        (1-normcdf(X1tau*par))))*ones(1,col+1)).*X1tau;
    
    nabla_tilde_theta_logf=tauvar_tot.*(normpdf(X1tau*par).*(educ-normcdf(X1tau*par))./...
        (normcdf(X1tau*par).*(1-normcdf(X1tau*par))))...
        -(mean((V*ones(1,col+1)).*X1tau)*inv(Heta)*nabla_eta_logf')';
    
    Htilde=var(nabla_tilde_theta_logf);
    
    epsilon1=2.8^2/(N*Htilde);
    
    %interpretation of epsilon in terms of marginal utility
    if female==0
        epsilon=.00872072^2;
        
    end
    
    if female==1
        epsilon=.00718376^2;
        
    end
    
    epsilon0=epsilon;%corresponding MC of subsidy=MC of consumption
    
    res_mmse=delta_theta+sum(nabla_eta_logf)/N*inv(Heta)*nabla_eta_delta...
        +(sum(nabla_tilde_theta_logf)/N)*inv(Htilde+(epsilon*N)^(-1))*nabla_tilde_theta_delta;
    
    disp('minimum-MSE')
    disp(res_mmse)
    
    % Confidence intervals
    
    disp('confidence intervals')
    
    % model-based, non-robust CI
    
    MM=(X1tau1'*(poor.*(normpdf(X1*par+par(1)*tauvar)))-X1'*(poor.*(normpdf(X1*par))))/sum(poor);
    
    aa=((((normpdf(X1*par).*(educ-normcdf(X1*par))./(normcdf(X1*par).*(1-normcdf(X1*par)))))*ones(1,col+1)).*X1)';
    
    Vect2=normpdf(X1*par).^2./(normcdf(X1*par).*(1-normcdf(X1*par)));
    
    Heta2=((Vect2*ones(1,col+1)).*X1)'*X1/N;
    
    vari=-(MM'*inv(Heta2)*aa)';
    
    std=sqrt(var(vari)/N);
    
    ci_inf=res-1.96*std;
    ci_sup=res+1.96*std;
    
    disp('model-based, non-robust')
    disp([ci_inf ci_sup])
    
    
    % model-based, bias
    
    bias=sqrt(epsilon)*(nabla_tilde_theta_delta-sum(vari.*nabla_tilde_theta_logf)/N);
    
    % model-based, robust CI
    
    ci_inf_rob=res-abs(bias)-1.96*std;
    ci_sup_rob=res+abs(bias)+1.96*std;
    
    disp('model-based, robust')
    disp([ci_inf_rob ci_sup_rob])
    
    % Minimum-MSE, robust CI
    
    influ=nabla_eta_logf*inv(Heta)*nabla_eta_delta...
        +nabla_tilde_theta_logf*inv(Htilde+(epsilon*N)^(-1))*nabla_tilde_theta_delta;
    
    
    bias_mmse=sqrt(epsilon)*(nabla_tilde_theta_delta-sum(influ.*nabla_tilde_theta_logf)/N);
    
    std_mmse=sqrt(var(influ)/N);
    
    ci_inf_mmse=res_mmse-abs(bias_mmse)-1.96*std_mmse;
    ci_sup_mmse=res_mmse+abs(bias_mmse)+1.96*std_mmse;
    
    disp('minimum-MSE')
    disp([ci_inf_mmse ci_sup_mmse])
    
    % Counterfactual 1: doubling the subsidy
    
    disp('doubling the subsidy')
    
    % estimates
    
    % model-based
    
    disp('point estimates')
    
    
    if female==1
        tauvar=(12/52)*(70*(grade==3)+80*(grade==4)+105*(grade==5)+135*(grade==6)+210*(grade==7)+235*(grade==8)+255*(grade==9));
    else
        tauvar=(12/52)*(70*(grade==3)+80*(grade==4)+105*(grade==5)+135*(grade==6)+200*(grade==7)+210*(grade==8)+225*(grade==9));
    end
    
    tauvar=2*tauvar;
    
    res=sum((normcdf(X1*par+par(1)*tauvar)-normcdf(X1*par)).*poor)/sum(poor);
    
    disp('model-based')
    disp(res)
    
    % Minimum-MSE
    
    delta_theta=sum((normcdf(X1*par+par(1)*tauvar)-normcdf(X1*par)).*poor)/sum(poor);
    
    X1tau1=X1;
    X1tau1(:,1)=X1(:,1)+tauvar;
    
    nabla_eta_delta=(X1tau1'*(poor.*(normpdf(X1*par+par(1)*tauvar)))-X1'*(poor.*(normpdf(X1*par))))/sum(poor);
    
    nabla_tilde_theta_delta=(tauvar'*(poor.*(normpdf(X1*par+par(1)*tauvar))))/sum(poor)...
        -mean((V*ones(1,col+1)).*X1tau)*inv(Heta)*nabla_eta_delta;
    
    nabla_eta_logf=((normpdf(X1tau*par).*(educ-normcdf(X1tau*par))./(normcdf(X1tau*par).*...
        (1-normcdf(X1tau*par))))*ones(1,col+1)).*X1tau;
    
    nabla_tilde_theta_logf=tauvar_tot.*(normpdf(X1tau*par).*(educ-normcdf(X1tau*par))./...
        (normcdf(X1tau*par).*(1-normcdf(X1tau*par))))...
        -(mean((V*ones(1,col+1)).*X1tau)*inv(Heta)*nabla_eta_logf')';
    
    res_mmse=delta_theta+sum(nabla_eta_logf)/N*inv(Heta)*nabla_eta_delta...
        +(sum(nabla_tilde_theta_logf)/N)*inv(Htilde+(epsilon*N)^(-1))*nabla_tilde_theta_delta;
    
    disp('minimum-MSE')
    disp(res_mmse)
    
    % Confidence intervals
    
    disp('confidence intervals')
    
    % model-based, non-robust CI
    
    MM=(X1tau1'*(poor.*(normpdf(X1*par+par(1)*tauvar)))-X1'*(poor.*(normpdf(X1*par))))/sum(poor);
    
    aa=((((normpdf(X1*par).*(educ-normcdf(X1*par))./(normcdf(X1*par).*(1-normcdf(X1*par)))))*ones(1,col+1)).*X1)';
    
    Vect2=normpdf(X1*par).^2./(normcdf(X1*par).*(1-normcdf(X1*par)));
    
    Heta2=((Vect2*ones(1,col+1)).*X1)'*X1/N;
    
    vari=-(MM'*inv(Heta2)*aa)';
    
    std=sqrt(var(vari)/N);
    
    % model-based, bias
    
    bias=sqrt(epsilon)*(nabla_tilde_theta_delta-sum(vari.*nabla_tilde_theta_logf)/N);
    
    % model-based, robust CI
    
    ci_inf_rob=res-abs(bias)-1.96*std;
    ci_sup_rob=res+abs(bias)+1.96*std;
    
    disp('model-based, robust')
    disp([ci_inf_rob ci_sup_rob])
    
    % Minimum-MSE, robust CI
    
    influ=nabla_eta_logf*inv(Heta)*nabla_eta_delta...
        +nabla_tilde_theta_logf*inv(Htilde+(epsilon*N)^(-1))*nabla_tilde_theta_delta;
    
    bias_mmse=sqrt(epsilon)*(nabla_tilde_theta_delta-sum(influ.*nabla_tilde_theta_logf)/N);
    
    std_mmse=sqrt(var(influ)/N);
    
    ci_inf_mmse=res_mmse-abs(bias_mmse)-1.96*std_mmse;
    ci_sup_mmse=res_mmse+abs(bias_mmse)+1.96*std_mmse;
    
    disp('minimum-MSE')
    disp([ci_inf_mmse ci_sup_mmse])
    
    % Counterfactual 2: unconditional transfer on all households
    
    disp('unconditional transfer')
    
    % estimates
    
    % model-based
    
    disp('point estimates')
    
    tauvar=5000/52;
    
    res=sum((normcdf(X1*par+(par(1)+par(2))*tauvar)-normcdf(X1*par)).*poor)/sum(poor);
    
    disp('model-based')
    disp(res)
    
    % Minimum-MSE
    
    delta_theta=sum((normcdf(X1*par+(par(1)+par(2))*tauvar)-normcdf(X1*par)).*poor)/sum(poor);
    
    X1tau1=X1;
    X1tau1(:,1)=X1(:,1)+tauvar;
    X1tau1(:,2)=X1(:,2)+tauvar;
    
    nabla_eta_delta=(X1tau1'*(poor.*(normpdf(X1*par+(par(1)+par(2))*tauvar)))-X1'*(poor.*(normpdf(X1*par))))/sum(poor);
    
    nabla_tilde_theta_delta=((tauvar*ones(N,1))'*(poor.*(normpdf(X1*par+(par(1)+par(2))*tauvar))))/sum(poor)...
        -mean((V*ones(1,col+1)).*X1tau)*inv(Heta)*nabla_eta_delta;
    
    nabla_eta_logf=((normpdf(X1tau*par).*(educ-normcdf(X1tau*par))./(normcdf(X1tau*par).*...
        (1-normcdf(X1tau*par))))*ones(1,col+1)).*X1tau;
    
    nabla_tilde_theta_logf=tauvar_tot.*(normpdf(X1tau*par).*(educ-normcdf(X1tau*par))./...
        (normcdf(X1tau*par).*(1-normcdf(X1tau*par))))...
        -(mean((V*ones(1,col+1)).*X1tau)*inv(Heta)*nabla_eta_logf')';
    
    res_mmse=delta_theta+sum(nabla_eta_logf)/N*inv(Heta)*nabla_eta_delta...
        +(sum(nabla_tilde_theta_logf)/N)*inv(Htilde+(epsilon*N)^(-1))*nabla_tilde_theta_delta;
    
    disp('minimum-MSE')
    disp(res_mmse)
    
    % Confidence intervals
    
    disp('confidence intervals')
    
    % model-based, non-robust CI
    
    MM=(X1tau1'*(poor.*(normpdf(X1*par+(par(1)+par(2))*tauvar)))-X1'*(poor.*(normpdf(X1*par))))/sum(poor);
    
    aa=((((normpdf(X1*par).*(educ-normcdf(X1*par))./(normcdf(X1*par).*(1-normcdf(X1*par)))))*ones(1,col+1)).*X1)';
    
    Vect2=normpdf(X1*par).^2./(normcdf(X1*par).*(1-normcdf(X1*par)));
    
    Heta2=((Vect2*ones(1,col+1)).*X1)'*X1/N;
    
    vari=-(MM'*inv(Heta2)*aa)';
    
    std=sqrt(var(vari)/N);
    
    % model-based, bias
    
    bias=sqrt(epsilon)*(nabla_tilde_theta_delta-sum(vari.*nabla_tilde_theta_logf)/N);
    
    % model-based, robust CI
    
    ci_inf_rob=res-abs(bias)-1.96*std;
    ci_sup_rob=res+abs(bias)+1.96*std;
    
    disp('model-based, robust')
    disp([ci_inf_rob ci_sup_rob])
    
    % Minimum-MSE, robust CI
    
    influ=nabla_eta_logf*inv(Heta)*nabla_eta_delta...
        +nabla_tilde_theta_logf*inv(Htilde+(epsilon*N)^(-1))*nabla_tilde_theta_delta;
    
    bias_mmse=sqrt(epsilon)*(nabla_tilde_theta_delta-sum(influ.*nabla_tilde_theta_logf)/N);
    
    std_mmse=sqrt(var(influ)/N);
    
    ci_inf_mmse=res_mmse-abs(bias_mmse)-1.96*std_mmse;
    ci_sup_mmse=res_mmse+abs(bias_mmse)+1.96*std_mmse;
    
    disp('minimum-MSE')
    disp([ci_inf_mmse ci_sup_mmse])
    
    % Graph Minimum-MSE estimator and CI (Figure 1, bottom panel)
    
    grid_eps=(0:.000001:.0002)';
    
    res_point=zeros(size(grid_eps,1),1);
    res_inf=zeros(size(grid_eps,1),1);
    res_sup=zeros(size(grid_eps,1),1);
    res_bias=zeros(size(grid_eps,1),1);
    res_std=zeros(size(grid_eps,1),1);
    
    % loop on epsilon
    for jp=1:size(grid_eps,1)
        
        epsilon=grid_eps(jp);
        
        if female==1
            tauvar=(12/52)*(70*(grade==3)+80*(grade==4)+105*(grade==5)+135*(grade==6)+210*(grade==7)+235*(grade==8)+255*(grade==9));
        else
            tauvar=(12/52)*(70*(grade==3)+80*(grade==4)+105*(grade==5)+135*(grade==6)+200*(grade==7)+210*(grade==8)+225*(grade==9));
        end
        
        X1tau=X1;
        X1tau(:,1)=X1(:,1)+tauvar_tot;
        
        Vect=normpdf(X1tau*par).^2./(normcdf(X1tau*par).*(1-normcdf(X1tau*par)));
        
        Heta=((Vect*ones(1,col+1)).*X1tau)'*X1tau/N;
        
        V=tauvar_tot.*normpdf(X1tau*par).^2./(normcdf(X1tau*par).*(1-normcdf(X1tau*par)));
        
        delta_theta=sum((normcdf(X1*par+par(1)*tauvar)-normcdf(X1*par)).*poor)/sum(poor);
        
        X1tau1=X1;
        X1tau1(:,1)=X1(:,1)+tauvar;
        
        nabla_eta_delta=(X1tau1'*(poor.*(normpdf(X1*par+par(1)*tauvar)))-X1'*(poor.*(normpdf(X1*par))))/sum(poor);
        
        nabla_tilde_theta_delta=(tauvar'*(poor.*(normpdf(X1*par+par(1)*tauvar))))/sum(poor)...
            -mean((V*ones(1,col+1)).*X1tau)*inv(Heta)*nabla_eta_delta;
        
        nabla_eta_logf=((normpdf(X1tau*par).*(educ-normcdf(X1tau*par))./(normcdf(X1tau*par).*...
            (1-normcdf(X1tau*par))))*ones(1,col+1)).*X1tau;
        
        nabla_tilde_theta_logf=tauvar_tot.*(normpdf(X1tau*par).*(educ-normcdf(X1tau*par))./...
            (normcdf(X1tau*par).*(1-normcdf(X1tau*par))))...
            -(mean((V*ones(1,col+1)).*X1tau)*inv(Heta)*nabla_eta_logf')';
        
        Htilde=var(nabla_tilde_theta_logf);
        
        % numerical approximation for small epsilon
        if epsilon>.000000001
            res_mmse=delta_theta+sum(nabla_eta_logf)/N*inv(Heta)*nabla_eta_delta...
                +(sum(nabla_tilde_theta_logf)/N)*inv(Htilde+(epsilon*N)^(-1))*nabla_tilde_theta_delta;
        else
            res_mmse=delta_theta+sum(nabla_eta_logf)/N*inv(Heta)*nabla_eta_delta;
        end
        
        res_point(jp)=res_mmse;
        
        % Confidence intervals
        
        warning('off')
        
        influ=nabla_eta_logf*inv(Heta)*nabla_eta_delta...
            +nabla_tilde_theta_logf*inv(Htilde+(epsilon*N)^(-1))*nabla_tilde_theta_delta;
        
        bias_mmse=sqrt(epsilon)*(nabla_tilde_theta_delta-sum(influ.*nabla_tilde_theta_logf)/N);
        
        std_mmse=sqrt(var(influ)/N);
        
        res_inf(jp,1)=res_mmse-abs(bias_mmse)-1.96*std_mmse;
        res_sup(jp,1)=res_mmse+abs(bias_mmse)+1.96*std_mmse;
        
        res_bias(jp,1)=abs(bias_mmse);
        res_std(jp,1)=std_mmse;
        
    end
    
    
    
    res=sum((normcdf(X1*par+par(1)*tauvar)-normcdf(X1*par)).*poor)/sum(poor);
    
    % Figure 1 (bottom panel)
    figure
    plot(grid_eps,res_point,'-','Color','b','Linewidth',3);
    hold on
    plot(grid_eps,res_inf,'--','Color','g','Linewidth',3);
    hold on
    plot(grid_eps,res_sup,'--','Color','g','Linewidth',3);
    hold on
    plot(grid_eps,res*ones(size(grid_eps,1),1),':','Color','k','Linewidth',3);
    axis([min(grid_eps) max(grid_eps) -.20 .35])
    vline(mean(epsilon1),'k-')
    vline(epsilon0,'k--')
    xlabel('epsilon','fontsize',14)
    ylabel('effect of PROGRESA','fontsize',14)
    hold off
   
end



