* Code for Bonhomme and Weidner (2021), "Minimizing Sensitivity to Model Misspecification" 
* This stata do file prepares the data to be used in the empirical application

clear 
cls

* enter your path here
*cd 

set matsize 5000

* the data set from Todd and Wolpin (2006)
use fin2a7v13c.dta, replace

* defines controls and treated
gen contvar=0
replace contvar=1 if (control==2 & year==98)|(year==97)

* log wages
gen logw=ln(weeklywage1)

* regression and predicted log wages
xi: reg logw i.agefin*i.sexr i.year i.entidad1 distance /*
*/ if work==1  & weeklywage1~=-9 & weeklywage1~=0  & sexr~=-9 /*
*/ & agefin~=-9 & distance~=-9 & contvar==1

predict logw_pred
scalar sig=e(rmse)

* predicted wages
gen W=exp(logw_pred+1/2*sig^2) 

* income
gen Y=parwklyinc1

/* Remark 1: only using treatment-control at the village level. Effect is heterogeneous by eligibility status (variable "pobre") */

/* Remark 2: imputing wages at the region-year level, using region and distance to the nearest city as determinants. Neither of them are included in the school choice equations. */
/* Todd and Wolpin (2008) use data on minimum wages at the village level, which we do not have. */ 
/* Todd and Wolpin (2006) allow explicitly for non-random selection into work, and use distance to the nearest city as an excluded covariate. */ 

/* sample selection */
sort entidad1 mpio local folio numero year

* drop missing data 
keep if Y~=-9999

keep if inschoolr~=-9

* age selection
keep if agefin>=12 & agefin<=15 

gen YW=Y+W

/* FEMALES */

* regression to produce model-based estimates
* the vector of estimates is used later in the matlab code 
xi: probit inschoolr Y YW i.agefin agemom agedad dissec i.year pobre hgc if contvar==1 & sexr==0
matrix eb_female=e(b)

preserve
drop _I*
keep if sexr==0

* save data (females)
save data_female, replace

outsheet using data_female, comma replace
restore

/* MALES */

* regression to produce model-based estimates
* the vector of estimates is used later in the matlab code 
xi: probit inschoolr Y YW i.agefin agemom agedad dissec i.year pobre hgc if contvar==1 & sexr==1
matrix eb_male=e(b)

drop _I*
keep if sexr==1

* save data (males)
save data_male, replace

outsheet using data_male, comma replace

* produce the parameter vectors to be used in the matlab code
matrix list eb_female
matrix list eb_male
